@@ -46,41 +46,36 @@
 #include "typedefs.h"
 #include "goo-window.h"
 #include "preferences.h"
-#include "song-info.h"
 #include "track-info.h"
-#include "goo-player-cd.h"
+#include "goo-player.h"
 #include "goo-cdrom.h"
 
 #define TOC_OFFSET 150
-#define GLADE_RIPPER_FILE "goobox.glade"
+#define GLADE_RIPPER_FILE "ripper_dialog.glade"
 #define DESTINATION_PERMISSIONS 0755
 #define PLS_PERMISSIONS 0644
 #define UPDATE_DELAY 400
-#define DEFAULT_OGG_QUALITY 0.3
+#define DEFAULT_OGG_QUALITY 0.5
 #define DEFAULT_FLAC_COMPRESSION 5
-#define DEFAULT_MP3_QUALITY 4
 #define BUFFER_SIZE 1024
 
 typedef struct {
 	GooWindow     *window;
-	char          *location;
+	char          *device;
 	char          *destination;
 	GooFileFormat  format;
-	char          *album;
-	char          *artist;
-	int            year;
-	char          *genre;
 	GList         *tracks;
-	int            tracks_n;
-	int            total_tracks;
+	int            n_tracks;
 	GList         *current_track;
 	int            current_track_n;
 	char          *ext;
 	GooCdrom      *cdrom;
+	AlbumInfo     *album;
 
-	GstElement    *rip_thread;
+	GstElement    *pipeline;
 	GstElement    *source;
 	GstElement    *encoder;
+	GstElement    *container;
 	GstElement    *sink;
 	GstPad        *source_pad;
 	GstFormat      track_format, sector_format;
@@ -91,12 +86,12 @@ typedef struct {
 	char          *current_file;
 	gboolean       ripping;
 	GTimer        *timer;
+	double         prev_remaining_time;
 
 	GladeXML      *gui;
 
 	GtkWidget     *dialog;
 	GtkWidget     *r_progress_progressbar;
-	GtkWidget     *r_progress_label;
 	GtkWidget     *r_track_label;
 } DialogData;
 
@@ -112,8 +107,6 @@ typedef enum vbr_mode_e {
   vbr_default=vbr_rh    /* change this to change the default VBR mode of LAME */
 } vbr_mode;
 
-static int mp3_bitrate[] = { 320, 256, 224, 192, 160, 128, 112, 96, 80, 64 };
-
 
 /* called when the main dialog is closed. */
 static void
@@ -128,9 +121,9 @@ destroy_cb (GtkWidget  *widget,
 	if (data->ripping && (data->current_file != NULL))
 		gnome_vfs_unlink (data->current_file);
 
-	if (data->rip_thread != NULL) {
-		gst_element_set_state (data->rip_thread, GST_STATE_NULL);
-		gst_object_unref (GST_OBJECT (data->rip_thread));
+	if (data->pipeline != NULL) {
+		gst_element_set_state (data->pipeline, GST_STATE_NULL);
+		gst_object_unref (GST_OBJECT (data->pipeline));
 	}
 
 	if (data->timer != NULL)
@@ -139,11 +132,8 @@ destroy_cb (GtkWidget  *widget,
 	goo_cdrom_unlock_tray (data->cdrom);
 	g_object_unref (data->cdrom);
 	
-	g_free (data->location);
+	g_free (data->device);
 	g_free (data->destination);
-	g_free (data->album);
-	g_free (data->artist);
-	g_free (data->genre);
 	g_free (data->current_file);
 	track_list_free (data->tracks);
 
@@ -165,7 +155,7 @@ rip_next_track (gpointer callback_data)
 	data->current_file = NULL;
 
 	gst_element_set_state (data->encoder, GST_STATE_NULL);
-	gst_element_set_state (data->rip_thread, GST_STATE_NULL);
+	gst_element_set_state (data->pipeline, GST_STATE_NULL);
 
 	track = data->current_track->data;
 	data->prev_tracks_sectors += track->sectors;
@@ -178,82 +168,101 @@ rip_next_track (gpointer callback_data)
 }
 
 
-static char *
-time_pretty_print (double s)
-{
-	char *r, *rr;
-	int   i;
-	
-	if (s < 0.0)
-		return g_strdup ("00:00");
-
-	i = (int) (floor (s + 0.5));
-	r = g_strdup_printf ("%2d:%2d", i / 60, i % 60);
-	for (rr = r; *rr != 0; rr++)
-		if (*rr == ' ')
-			*rr = '0';
-
-	return r;
-}
-
-
 static gboolean
 update_ui (gpointer callback_data)
 {
 	DialogData *data = callback_data;
+	int         ripped_tracks;
 	double      fraction;
-	double      e, r;
-	char       *elapsed, *s_e, *s_r;
+	char       *msg, *time_left;
 
-	fraction = ((double) (data->current_track_sectors + data->prev_tracks_sectors)) / (double) data->total_sectors;
+	ripped_tracks = data->current_track_sectors + data->prev_tracks_sectors;
+	fraction = (double) ripped_tracks / data->total_sectors;
 	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (data->r_progress_progressbar), fraction);
 
-	e = g_timer_elapsed (data->timer, NULL);
-	r = (e / fraction) - e;
-	s_e = time_pretty_print (e);
-	s_r = time_pretty_print (r);
-	elapsed = g_strdup_printf (_("Elapsed: %s / Remaining: %s"), s_e, s_r);
-	gtk_progress_bar_set_text  (GTK_PROGRESS_BAR (data->r_progress_progressbar), elapsed);
-	g_free (s_e);
-	g_free (s_r);
-	g_free (elapsed);
+	/* wait a bit before predicting the remaining time. */
+	if ((fraction < 10e-3) || (ripped_tracks < 250)) {
+		time_left = NULL;
+		data->prev_remaining_time = -1.0;
+	} 
+	else {  
+		double elapsed, remaining_time;
+		int    minutes, seconds;
+		
+		elapsed = g_timer_elapsed (data->timer, NULL);
+		remaining_time = (elapsed / fraction) - elapsed;	
+		minutes = (int) (floor (remaining_time + 0.5)) / 60;
+		seconds = (int) (floor (remaining_time + 0.5)) % 60;
+		
+		if ((data->prev_remaining_time > 0.0) 
+		    && ( fabs (data->prev_remaining_time - remaining_time) > 20.0))
+			time_left = NULL;
+		else if (minutes > 59) 
+			time_left = g_strdup_printf (_("(%d:%02d:%02d Remaining)"), minutes / 60, minutes % 60, seconds);
+		else 
+			time_left = g_strdup_printf (_("(%d:%02d Remaining)"), minutes, seconds);
+			
+		data->prev_remaining_time = remaining_time;
+	}
+	
+	msg = g_strdup_printf (_("Extracting track: %d of %d %s"), data->current_track_n, data->n_tracks, (time_left != NULL ? time_left : ""));
+	gtk_progress_bar_set_text  (GTK_PROGRESS_BAR (data->r_progress_progressbar), msg);
+
+	g_free (msg);
+	g_free (time_left);
 
 	return FALSE;
 }
 
 
 static void
-encoder_eos_cb (GstBin     *bin, 
-		DialogData *data)
+pipeline_eos_cb (GstBus     *bus, 
+		 GstMessage *message, 
+		 gpointer    callback_data)
 {
+	DialogData *data = callback_data;
+	
 	if (data->update_handle != 0) {
 		g_source_remove (data->update_handle);
 		data->update_handle = 0;
 	}
-	g_idle_add (rip_next_track, data);
+	g_idle_add (rip_next_track, data);	
+}
+
+
+static void
+pipeline_error_cb (GstBus     *bus, 
+		   GstMessage *message, 
+		   gpointer    callback_data)
+{
+	DialogData *data = callback_data;
+	GError     *error;
+
+	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
+	gtk_widget_hide (data->dialog);
+
+	gst_message_parse_error (message, &error, NULL);
+	_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->window), _("Could not extract the tracks"), &error);	
+
+	gtk_widget_destroy (data->dialog);	
 }
 
 
 static gboolean
 update_progress_cb (gpointer callback_data)
 {
-	DialogData  *data = callback_data;
-	TrackInfo   *track = data->current_track->data;
-	gint64       from_sector;
-	guint64      sector = 0;
-	gboolean     ret;
+	DialogData *data = callback_data;
+	gint64      sector = 0;
 
 	if (data->update_handle != 0) {
 		g_source_remove (data->update_handle);
 		data->update_handle = 0;
 	}
 	
-	ret = gst_pad_query (data->source_pad, GST_QUERY_POSITION, &data->sector_format, &sector);
-	if (!ret)
+	if (! gst_pad_query_position (data->source_pad, &data->sector_format, &sector))
 		return FALSE;
 	
-	from_sector = track->from_sector - TOC_OFFSET;
-	data->current_track_sectors = sector - from_sector;
+	data->current_track_sectors = sector;
 	g_idle_add (update_ui, data);
 
 	data->update_handle = g_timeout_add (UPDATE_DELAY, update_progress_cb, data);
@@ -265,72 +274,91 @@ update_progress_cb (gpointer callback_data)
 static void
 create_pipeline (DialogData *data)
 {
+	GstBus     *bus;
+	GstElement *audioconvert;
+	GstElement *audioresample;	
+	GstElement *queue;
 	float       ogg_quality;
 	int         flac_compression;
-	int         mp3_quality;
-	GParamSpec *pspec;
-
-	data->rip_thread = gst_thread_new ("rip_thread");
-		
-	data->source = gst_element_factory_make ("cdparanoia", "cdparanoia");
+	
+	data->pipeline = gst_pipeline_new ("pipeline");
+	
+	/*data->source = gst_element_make_from_uri (GST_URI_SRC, "cdda://1", "source");*/
+	data->source = gst_element_factory_make ("cdparanoiasrc", "source");
 	g_object_set (G_OBJECT (data->source), 
-		      "device", data->location,
+		      "device", data->device, 
+		      "read-speed", G_MAXINT,
 		      NULL);
 
+	audioconvert = gst_element_factory_make ("audioconvert", "convert");
+    	audioresample = gst_element_factory_make ("audioresample", "resample");
+    	
+	queue = gst_element_factory_make ("queue", "queue");
+	g_object_set (queue, "max-size-time", 120 * GST_SECOND, NULL);
+
 	switch (data->format) {
 	case GOO_FILE_FORMAT_OGG:	
-		data->encoder = gst_element_factory_make (OGG_ENCODER, "encoder");
 		data->ext = "ogg";
-		ogg_quality = eel_gconf_get_float (PREF_ENCODER_OGG_QUALITY, DEFAULT_OGG_QUALITY) + 0.05;
+		
+		data->encoder = gst_element_factory_make (OGG_ENCODER, "encoder");
+		ogg_quality = eel_gconf_get_float (PREF_ENCODER_OGG_QUALITY, DEFAULT_OGG_QUALITY);
 		g_object_set (data->encoder,
 			      "quality", ogg_quality,
 			      NULL);
+		
+		data->container = gst_element_factory_make ("oggmux", "container");
 		break;
 
 	case GOO_FILE_FORMAT_FLAC:
-		data->encoder = gst_element_factory_make (FLAC_ENCODER, "encoder");
 		data->ext = "flac";
+		
+		data->encoder = data->container = gst_element_factory_make (FLAC_ENCODER, "encoder");
 		flac_compression = eel_gconf_get_integer (PREF_ENCODER_FLAC_COMPRESSION, DEFAULT_FLAC_COMPRESSION);
 		g_object_set (data->encoder,
 			      "quality", flac_compression,
 			      NULL);
 		break;
 
-	case GOO_FILE_FORMAT_MP3:
-		data->encoder = gst_element_factory_make (MP3_ENCODER, "encoder");
-		data->ext = "mp3";
-		mp3_quality = eel_gconf_get_integer (PREF_ENCODER_MP3_QUALITY, DEFAULT_MP3_QUALITY);
-		pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (data->encoder), "vbr_quality");
-		if (pspec != NULL) {
-			g_object_set (data->encoder,
-				      "vbr", vbr_mtrh,
-				      "vbr_quality", mp3_quality,
-				      NULL);
-			g_param_spec_unref (pspec);
-		} else 
-			g_object_set (data->encoder,
-				      "bitrate", mp3_bitrate[mp3_quality],
-				      NULL);
-		break;
-
 	case GOO_FILE_FORMAT_WAVE:
-		data->encoder = gst_element_factory_make (WAVE_ENCODER, "encoder");
 		data->ext = "wav";
+		
+		data->encoder = data->container = gst_element_factory_make (WAVE_ENCODER, "encoder");
 		break;
 	}
-	g_signal_connect (data->encoder, 
-			  "eos", 
-			  G_CALLBACK (encoder_eos_cb), 
-			  data);
 
-	data->sink = gst_element_factory_make ("gnomevfssink", "filesink");
+	data->sink = gst_element_factory_make ("gnomevfssink", "sink");
 
-	gst_bin_add_many (GST_BIN (data->rip_thread), data->source, data->encoder, data->sink, NULL);
-	gst_element_link_many (data->source, data->encoder, data->sink, NULL);
+	if (data->encoder == data->container) {
+		gst_bin_add_many (GST_BIN (data->pipeline), data->source, queue, 
+				  audioconvert, audioresample, data->encoder, 
+				  data->sink, NULL);
+		gst_element_link_many (data->source, queue, audioconvert, 
+				       audioresample, data->encoder, 
+				       data->sink, NULL);
+	}
+	else {
+		gst_bin_add_many (GST_BIN (data->pipeline), data->source, queue, 
+				  audioconvert, audioresample, data->encoder,
+				  data->container, data->sink, NULL);
+		gst_element_link_many (data->source, queue, audioconvert, 
+				       audioresample, data->encoder, 
+				       data->container, data->sink, NULL);
+	}
 
 	data->track_format = gst_format_get_by_nick ("track");
 	data->sector_format = gst_format_get_by_nick ("sector");
 	data->source_pad = gst_element_get_pad (data->source, "src");
+
+	bus =  gst_element_get_bus (data->pipeline);
+	gst_bus_add_signal_watch (bus);
+	g_signal_connect (G_OBJECT (bus), 
+			  "message::error", 
+			  G_CALLBACK (pipeline_error_cb), 
+			  data);
+	g_signal_connect (G_OBJECT (bus), 
+			  "message::eos", 
+			  G_CALLBACK (pipeline_eos_cb), 
+			  data);
 }
 
 
@@ -341,13 +369,13 @@ get_destination_folder (DialogData *data)
 	char *album_filename;
 	char *result;
 
-	artist_filename = tracktitle_to_filename (data->artist);
-	album_filename = tracktitle_to_filename (data->album);
+	artist_filename = tracktitle_to_filename (data->album->artist);
+	album_filename = tracktitle_to_filename (data->album->title);
 	
-	result = g_build_filename (data->destination,
-				   artist_filename,
-				   album_filename,
-				   NULL);
+	result = g_strconcat (data->destination, G_DIR_SEPARATOR_S,
+			      artist_filename, G_DIR_SEPARATOR_S,
+			      album_filename,
+			      NULL);
 
 	g_free (artist_filename);
 	g_free (album_filename);
@@ -373,7 +401,7 @@ done_dialog_response_cb (GtkDialog  *dialog,
 		char   *url    = NULL;
 		
 		scheme = gnome_vfs_get_uri_scheme (folder);
-		if ((scheme == NULL) || (scheme == ""))
+		if ((scheme == NULL) || (strcmp (scheme, "") == 0))
 			url = gnome_vfs_get_uri_from_local_path (folder);
 		else
 			url = g_strdup (folder);
@@ -412,7 +440,7 @@ get_track_filename (TrackInfo  *track,
 	char *filename, *track_filename;
 
 	filename = tracktitle_to_filename (track->title);
-	track_filename = g_strdup_printf ("%s - %s.%s", zero_padded (track->number + 1), filename, ext);
+	track_filename = g_strdup_printf ("%s.%%20%s.%s", zero_padded (track->number + 1), filename, ext);
 	g_free (filename);
 
 	return track_filename;
@@ -428,7 +456,7 @@ save_playlist (DialogData *data)
 	GnomeVFSHandle *handle;
 
 	folder = get_destination_folder (data);
-	album_filename = tracktitle_to_filename (data->album);
+	album_filename = tracktitle_to_filename (data->album->title);
 	filename = g_strconcat ("file://", folder, "/", album_filename, ".pls", NULL);
 	g_free (album_filename);
 
@@ -451,7 +479,7 @@ save_playlist (DialogData *data)
 				 strlen (buffer),
 				 NULL);
 
-		sprintf (buffer, "NumberOfEntries=%d\n", data->tracks_n);
+		sprintf (buffer, "NumberOfEntries=%d\n", data->n_tracks);
 		gnome_vfs_write (handle,
 				 buffer,
 				 strlen (buffer),
@@ -466,19 +494,23 @@ save_playlist (DialogData *data)
 		for (scan = data->tracks; scan; scan = scan->next) {
 			TrackInfo *track = scan->data;
 			char      *track_filename;
+			char      *unescaped;
 
 			n++;
 
 			track_filename = get_track_filename (track, data->ext);
+			unescaped = gnome_vfs_unescape_string (track_filename, "");
 			
-			sprintf (buffer, "File%d=%s\n", n, track_filename);
+			sprintf (buffer, "File%d=%s\n", n, unescaped);
 			gnome_vfs_write (handle,
 					 buffer,
 					 strlen (buffer),
 					 NULL);
+			
+			g_free (unescaped);		 
 			g_free (track_filename);
 
-			sprintf (buffer, "Title%d=%s - %s\n", n, data->artist, track->title);
+			sprintf (buffer, "Title%d=%s - %s\n", n, data->album->artist, track->title);
 			gnome_vfs_write (handle,
 					 buffer,
 					 strlen (buffer),
@@ -506,7 +538,6 @@ rip_current_track (DialogData *data)
 	char           *msg;
 	char           *filename;
 	char           *folder;
-	char           *e_title;
 	GstEvent       *event;
 	GnomeVFSResult  result;
 
@@ -517,6 +548,7 @@ rip_current_track (DialogData *data)
 			save_playlist (data);
 
 		data->ripping = FALSE;
+		gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
 		gtk_widget_hide (data->dialog);
 
 		d = _gtk_ok_dialog_with_checkbutton_new (GTK_WINDOW (data->window),
@@ -534,19 +566,11 @@ rip_current_track (DialogData *data)
 
 		return;
 	}
-		
-	track = data->current_track->data;
-	msg = g_strdup_printf (_("Extracting track %d of %d: %s"), 
-			       data->current_track_n,
-			       data->tracks_n,
-			       "");
-	gtk_label_set_text (GTK_LABEL (data->r_progress_label), msg);
-	g_free (msg);
 
-	e_title = g_markup_escape_text (track->title, -1);
-	msg = g_strconcat ("<b>", e_title, "</b>", NULL);
+	track = data->current_track->data;
+
+	msg = g_markup_printf_escaped (_("<i>Extracting \"%s\"</i>"), track->title); 
 	gtk_label_set_markup (GTK_LABEL (data->r_track_label), msg);
-	g_free (e_title);
 	g_free (msg);
 
 	/* Set the filename */
@@ -558,8 +582,8 @@ rip_current_track (DialogData *data)
 
 		utf8_folder = g_locale_to_utf8 (folder, -1, 0, 0, 0);
 		_gtk_error_dialog_run (GTK_WINDOW (data->window),
-				       "Could not extract tracks",
-				       "Could not create folder %s\n\n%s",
+				       _("Could not extract tracks"),
+				       _("Could not create folder \"%s\"\n\n%s"),
 				       utf8_folder,
 				       gnome_vfs_result_to_string (result));
 		g_free (utf8_folder);
@@ -567,9 +591,9 @@ rip_current_track (DialogData *data)
 		return;
 	}
 
-	filename = get_track_filename (track, data->ext);
 	g_free (data->current_file);
-	data->current_file = g_build_filename (folder, filename, NULL);
+	filename = get_track_filename (track, data->ext);
+	data->current_file = g_strconcat (folder, G_DIR_SEPARATOR_S, filename, NULL);
 	g_free (filename);
 	g_free (folder);
 
@@ -578,41 +602,44 @@ rip_current_track (DialogData *data)
 	gst_element_set_state (data->sink, GST_STATE_NULL);
 	g_object_set (G_OBJECT (data->sink), "location", data->current_file, NULL);
 
-	/* Set song tags. */
+	/* Set track tags. */
 
 	if (GST_IS_TAG_SETTER (data->encoder)) {
-		gst_tag_setter_add (GST_TAG_SETTER (data->encoder),   
-				    GST_TAG_MERGE_APPEND,
-				    GST_TAG_TITLE, track->title,
-				    GST_TAG_ARTIST, data->artist,
-				    GST_TAG_ALBUM, data->album,
-				    GST_TAG_TRACK_NUMBER, (guint) track->number + 1,
-				    GST_TAG_TRACK_COUNT, (guint) data->total_tracks,
-				    GST_TAG_DURATION, (guint64) track->length * GST_SECOND, 
-				    GST_TAG_COMMENT, _("Ripped with Goobox"),
-				    GST_TAG_ENCODER, PACKAGE_NAME,
-				    GST_TAG_ENCODER_VERSION, PACKAGE_VERSION,
-				    NULL);
-		if (data->genre != NULL)
-			gst_tag_setter_add (GST_TAG_SETTER (data->encoder),   
-					    GST_TAG_MERGE_APPEND,
-					    GST_TAG_GENRE, data->genre,
-					    NULL);
-		if (data->year != 0) {
-			GDate *d = g_date_new_dmy (1, 1, data->year);
-			gst_tag_setter_add (GST_TAG_SETTER (data->encoder),   
-					    GST_TAG_MERGE_APPEND,
-					    GST_TAG_DATE,  g_date_get_julian (d),
-					    NULL);
-			g_date_free (d);
-		}
+		gst_tag_setter_add_tags (GST_TAG_SETTER (data->encoder),   
+				         GST_TAG_MERGE_REPLACE,
+				         GST_TAG_TITLE, track->title,
+				         GST_TAG_ARTIST, data->album->artist,
+				         GST_TAG_ALBUM, data->album->title,
+				         GST_TAG_TRACK_NUMBER, (guint) track->number + 1,
+				         GST_TAG_TRACK_COUNT, (guint) data->n_tracks,
+				         GST_TAG_DURATION, (guint64) track->length * GST_SECOND, 
+				         GST_TAG_COMMENT, _("Ripped with CD Player"),
+				         GST_TAG_ENCODER, PACKAGE_NAME,
+				         GST_TAG_ENCODER_VERSION, PACKAGE_VERSION,
+				         NULL);
+		if (data->album->genre != NULL)
+			gst_tag_setter_add_tags (GST_TAG_SETTER (data->encoder),   
+					         GST_TAG_MERGE_REPLACE,
+					         GST_TAG_GENRE, data->album->genre,
+					         NULL);
+		if (g_date_valid (data->album->release_date)) 
+			gst_tag_setter_add_tags (GST_TAG_SETTER (data->encoder),   
+					         GST_TAG_MERGE_REPLACE,
+					         GST_TAG_DATE, data->album->release_date,
+					         NULL);
 	}
 		
 	/* Seek to track. */
 
-	gst_element_set_state (data->rip_thread, GST_STATE_PAUSED);
-	event = gst_event_new_segment_seek (data->track_format | GST_SEEK_METHOD_SET | GST_SEEK_FLAG_FLUSH, track->number, track->number + 1);
-	if (!gst_pad_send_event (data->source_pad, event)) {
+	gst_element_set_state (data->pipeline, GST_STATE_PAUSED);
+	event = gst_event_new_seek (1.0, 
+				    data->track_format,
+				    GST_SEEK_FLAG_FLUSH,
+				    GST_SEEK_TYPE_SET,
+				    track->number,
+				    GST_SEEK_TYPE_SET,
+				    track->number + 1);
+	if (! gst_pad_send_event (data->source_pad, event)) {
 		g_warning ("seek failed");
 		return;
 	}
@@ -622,7 +649,7 @@ rip_current_track (DialogData *data)
 	data->current_track_sectors = 0;
 	data->update_handle = g_timeout_add (UPDATE_DELAY, update_progress_cb, data);
 
-	gst_element_set_state (data->rip_thread, GST_STATE_PLAYING);
+	gst_element_set_state (data->pipeline, GST_STATE_PLAYING);
 }
 
 
@@ -634,42 +661,32 @@ start_ripper (DialogData *data)
 	data->ripping = TRUE;
 
 	goo_cdrom_lock_tray (data->cdrom);
+	create_pipeline (data);
 
 	data->prev_tracks_sectors = 0;
 	data->total_sectors = 0;
 	for (scan = data->tracks; scan; scan = scan->next) {
 		TrackInfo *track = scan->data;
+		
 		data->total_sectors += track->sectors;
 	}
-	create_pipeline (data);
 	data->current_track_n = 1;
 	data->current_track = data->tracks;
 
-	g_timer_start (data->timer);
-
+	g_timer_start (data->timer);	
 	rip_current_track (data);
 }
 
 
-
-
-
 /* create the main dialog. */
 void
-dlg_ripper (GooWindow     *window,
-	    const char    *location,
-	    const char    *destination,
-	    GooFileFormat  format,
-	    const char    *album,
-	    const char    *artist,
-	    int            year,
-	    const char    *genre,
-	    int            total_tracks,
-	    GList         *tracks)
+dlg_ripper (GooWindow *window,
+	    GList     *tracks)
 {
 	DialogData  *data;
 	GtkWidget   *btn_cancel;
-
+	GooPlayer   *player;
+	
 	data = g_new0 (DialogData, 1);
 	data->window = window;
 	data->gui = glade_xml_new (GOO_GLADEDIR "/" GLADE_RIPPER_FILE, NULL, NULL);
@@ -679,37 +696,41 @@ dlg_ripper (GooWindow     *window,
                 return;
         }
 
-	data->location = g_strdup (location);
-	data->destination = g_strdup (destination);
-	data->format = format;
-	data->album = g_strdup (album);
-	data->artist = g_strdup (artist);
-	data->year = year;
-	if (genre != NULL)
-		data->genre = g_strdup (genre);
-	data->tracks = track_list_dup (tracks);
-	data->tracks_n = g_list_length (data->tracks);
-	data->total_tracks = total_tracks;
+	goo_window_stop (window);
+	player = goo_window_get_player (window);
+
+	data->destination = eel_gconf_get_path (PREF_EXTRACT_DESTINATION, "");
+	if ((data->destination == NULL) || (strcmp (data->destination, "") == 0)) { 
+		char *tmp;
+		
+		tmp = xdg_user_dir_lookup ("MUSIC");
+		data->destination = get_uri_from_local_path (tmp);
+		g_free (tmp);
+	}
+	data->device = g_strdup (goo_player_get_device (player));
+	data->format = pref_get_file_format ();
+	data->album = goo_player_get_album (player);
+	if (tracks == NULL)
+		data->tracks = track_list_dup (data->album->tracks);
+	else
+		data->tracks = track_list_dup (tracks);
+	data->n_tracks = g_list_length (data->tracks);
+	
 	data->update_handle = 0;
 	data->timer = g_timer_new ();
-	data->cdrom = goo_cdrom_new (location);
+	data->cdrom = goo_cdrom_new (data->device);
 
 	/* Get the widgets. */
 
 	data->dialog = glade_xml_get_widget (data->gui, "ripper_dialog");
 	data->r_progress_progressbar = glade_xml_get_widget (data->gui, "r_progress_progressbar");
-	data->r_progress_label = glade_xml_get_widget (data->gui, "r_progress_label");
 	data->r_track_label = glade_xml_get_widget (data->gui, "r_track_label");
 	btn_cancel = glade_xml_get_widget (data->gui, "r_cancelbutton");
 
 	/* Set widgets data. */
 
-#ifdef HAVE_GTK_2_5
-	gtk_label_set_ellipsize (GTK_LABEL (data->r_progress_label),
-				 PANGO_ELLIPSIZE_END);
 	gtk_label_set_ellipsize (GTK_LABEL (data->r_track_label),
 				 PANGO_ELLIPSIZE_END);
-#endif
 
 	/* Set the signals handlers. */
 
