@@ -3,7 +3,7 @@
 /*
  *  Goo
  *
- *  Copyright (C) 2004 Free Software Foundation, Inc.
+ *  Copyright (C) 2004, 2007 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,19 +32,23 @@
 
 #define SPACING 0
 #define TITLE1_FORMAT "<span size='large' weight='bold'>%s</span>"
-#define TITLE2_FORMAT "%s"
-#define TITLE3_FORMAT "<i>%s</i>"
+#define TITLE2_FORMAT "<i>%s</i>"
+#define TITLE3_FORMAT "%s"
 #define TIME_FORMAT "%s"
 #define PLAYING_FORMAT "[ <small>%s</small> ]"
 #define SCALE_WIDTH 150
 #define COVER_SIZE 80
-#define MIN_WIDTH 440
-#define MIN_CHARS 33
+#define TRAY_COVER_SIZE 80
+#define MIN_WIDTH 420
+#define MIN_TOOLTIP_WIDTH 300
+#define MAX_TOOLTIP_WIDTH 400
+#define MIN_CHARS 45
 #define UPDATE_TIMEOUT 50
 
 struct _GooPlayerInfoPrivateData {
 	GooWindow   *window;
-	GooPlayer   *player;
+	gboolean     interactive;
+	GtkWidget   *cover_frame;
 	GtkWidget   *title1_label;
 	GtkWidget   *title2_label;
 	GtkWidget   *title3_label;
@@ -53,14 +57,18 @@ struct _GooPlayerInfoPrivateData {
 	GtkWidget   *time_scale;
 	GtkWidget   *cover_image;
 	GtkWidget   *cover_button;
-	GtkWidget   *cover_sep;
+	GtkWidget   *status_image;
+	GtkWidget   *notebook;
 	GtkTooltips *tips;
 	char         current_time[64];
 	char         total_time[64];
 	char         time[64];
-	gint64       song_length;
+	gint64       track_length;
 	gboolean     dragging;
 	guint        update_id;
+	
+	double       fraction;
+	guint        update_progress_timeout;
 };
 
 enum {
@@ -69,7 +77,6 @@ enum {
         LAST_SIGNAL
 };
 
-static GtkHBoxClass *parent_class = NULL;
 static guint goo_player_info_signals[LAST_SIGNAL] = { 0 };
 
 enum { TARGET_URL };
@@ -78,46 +85,27 @@ static GtkTargetEntry target_table[] = {
 };
 static guint n_targets = sizeof (target_table) / sizeof (target_table[0]);
 
-static void goo_player_info_class_init  (GooPlayerInfoClass *class);
-static void goo_player_info_init        (GooPlayerInfo *player);
 static void goo_player_info_finalize    (GObject *object);
 
+#define GOO_PLAYER_INFO_GET_PRIVATE_DATA(object) \
+	(G_TYPE_INSTANCE_GET_PRIVATE ((object), GOO_TYPE_PLAYER_INFO, GooPlayerInfoPrivateData))
 
-GType
-goo_player_info_get_type ()
-{
-        static GType type = 0;
-
-        if (! type) {
-                GTypeInfo type_info = {
-			sizeof (GooPlayerInfoClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) goo_player_info_class_init,
-			NULL,
-			NULL,
-			sizeof (GooPlayerInfo),
-			0,
-			(GInstanceInitFunc) goo_player_info_init
-		};
-
-		type = g_type_register_static (GTK_TYPE_HBOX,
-					       "GooPlayerInfo",
-					       &type_info,
-					       0);
-	}
-
-        return type;
-}
-
+G_DEFINE_TYPE (GooPlayerInfo, goo_player_info, GTK_TYPE_HBOX)
 
 static void
 goo_player_info_size_request (GtkWidget      *widget,
 			      GtkRequisition *requisition)
 {	
-	if (GTK_WIDGET_CLASS (parent_class)->size_request)
-		(* GTK_WIDGET_CLASS (parent_class)->size_request) (widget, requisition);
-	requisition->width = MAX (requisition->width, MIN_WIDTH);
+	GooPlayerInfo *info;
+	
+	if (GTK_WIDGET_CLASS (goo_player_info_parent_class)->size_request)
+		(* GTK_WIDGET_CLASS (goo_player_info_parent_class)->size_request) (widget, requisition);
+	
+	info = (GooPlayerInfo *) widget;
+	if (info->priv->interactive)
+		requisition->width = MAX (requisition->width, MIN_WIDTH);
+	else
+		requisition->width = MIN (MAX (requisition->width, MIN_TOOLTIP_WIDTH), MAX_TOOLTIP_WIDTH);
 }
 
 
@@ -127,8 +115,6 @@ goo_player_info_class_init (GooPlayerInfoClass *class)
         GObjectClass   *gobject_class;
 	GtkWidgetClass *widget_class;
 
-        parent_class = g_type_class_peek_parent (class);
-
 	goo_player_info_signals[COVER_CLICKED] =
                 g_signal_new ("cover_clicked",
 			      G_TYPE_FROM_CLASS (class),
@@ -154,6 +140,8 @@ goo_player_info_class_init (GooPlayerInfoClass *class)
 
 	widget_class = GTK_WIDGET_CLASS (class);
 	widget_class->size_request = goo_player_info_size_request;
+
+	g_type_class_add_private (class, sizeof (GooPlayerInfoPrivateData));
 }
 
 
@@ -223,23 +211,21 @@ static void
 time_scale_value_changed_cb (GtkRange      *range,
 			     GooPlayerInfo *info)
 {
-	double new_value = gtk_range_get_value (range);
-	int    seconds = (int) (new_value * info->priv->song_length);
+	double new_value;
+	gint64 current_time;
+	
+	new_value = gtk_range_get_value (range);
+	current_time = info->priv->track_length * (new_value / 100.0);
+	set_time_string (info->priv->current_time, current_time);
+	sprintf (info->priv->time, _("%s / %s"), info->priv->total_time, info->priv->current_time);
+	set_time (info, info->priv->time);
 
-	if (info->priv->dragging) {
-		GooPlayerInfoPrivateData *priv = info->priv;
-		gint64 current_time;
-
-		current_time = priv->song_length * (new_value / 100.0);
-
-		set_time_string (priv->current_time, current_time);
-		sprintf (priv->time, _("%s / %s"), priv->total_time, priv->current_time);
-		set_time (info, priv->time);
-
-		return;
+	if (! info->priv->dragging) {
+		int seconds;
+		
+		seconds = (int) (new_value * info->priv->track_length);
+		g_signal_emit (info, goo_player_info_signals[SKIP_TO], 0, seconds);
 	}
-
-	g_signal_emit (info, goo_player_info_signals[SKIP_TO], 0, seconds);
 }
 
 
@@ -256,7 +242,7 @@ update_time_label_cb (gpointer data)
 		priv->update_id = 0;
 	}
 	
-	current_time = priv->song_length * new_value;
+	current_time = priv->track_length * new_value;
 	set_time_string (priv->current_time, current_time);
 	sprintf (priv->time, _("%s / %s"), priv->current_time, priv->total_time);
 	set_time (info, priv->time);
@@ -422,11 +408,17 @@ cover_button_drag_data_received  (GtkWidget          *widget,
 
 static void 
 goo_player_info_init (GooPlayerInfo *info)
+{
+	info->priv = GOO_PLAYER_INFO_GET_PRIVATE_DATA (info);
+}
+
+
+static void 
+goo_player_info_construct (GooPlayerInfo *info)
 {
 	GooPlayerInfoPrivateData *priv;
 	GtkWidget *vbox, *time_box;
 
-	info->priv = g_new0 (GooPlayerInfoPrivateData, 1);
 	priv = info->priv;
 
 	priv->dragging = FALSE;
@@ -434,6 +426,7 @@ goo_player_info_init (GooPlayerInfo *info)
 	priv->total_time[0] = '\0';
 	priv->update_id = 0;
 
+	GTK_WIDGET_UNSET_FLAGS (info, GTK_CAN_FOCUS);
 	GTK_BOX (info)->spacing = SPACING;
 	GTK_BOX (info)->homogeneous = FALSE;
 
@@ -448,13 +441,12 @@ goo_player_info_init (GooPlayerInfo *info)
 
 	priv->title2_label = gtk_label_new (NULL);
 	gtk_misc_set_alignment (GTK_MISC (priv->title2_label), 0.0, 0.5);
-	gtk_label_set_selectable (GTK_LABEL (priv->title2_label), TRUE);
+	gtk_label_set_selectable (GTK_LABEL (priv->title2_label), priv->interactive);
 
 	priv->title3_label = gtk_label_new (NULL);
 	gtk_misc_set_alignment (GTK_MISC (priv->title3_label), 0.0, 0.5);
-	gtk_label_set_selectable (GTK_LABEL (priv->title3_label), TRUE);
+	gtk_label_set_selectable (GTK_LABEL (priv->title3_label), priv->interactive);
 
-#ifdef HAVE_GTK_2_5
 	gtk_label_set_ellipsize (GTK_LABEL (priv->title1_label),
 				 PANGO_ELLIPSIZE_END);
 	gtk_label_set_width_chars (GTK_LABEL (priv->title2_label), MIN_CHARS);
@@ -466,7 +458,6 @@ goo_player_info_init (GooPlayerInfo *info)
 	gtk_label_set_ellipsize (GTK_LABEL (priv->title3_label),
 				 PANGO_ELLIPSIZE_END);
 	gtk_label_set_width_chars (GTK_LABEL (priv->title3_label), MIN_CHARS);
-#endif
 
 	/* Time */
 
@@ -491,40 +482,72 @@ goo_player_info_init (GooPlayerInfo *info)
 
 	/* Image */
 
-	priv->cover_button = gtk_button_new ();
-	gtk_button_set_relief (GTK_BUTTON (priv->cover_button),
-			       GTK_RELIEF_NONE);
-	gtk_tooltips_set_tip (priv->tips,
-			      GTK_WIDGET (priv->cover_button),
-			      _("Click here to choose a cover for this CD"),
-			      NULL);
-	g_signal_connect (G_OBJECT (priv->cover_button),
-			  "clicked",
-			  G_CALLBACK (cover_button_clicked_cb),
-			  info);
 
-	gtk_drag_dest_set (priv->cover_button,
-			   GTK_DEST_DEFAULT_ALL,
-			   target_table, n_targets,
-			   GDK_ACTION_COPY);
-	
-	g_signal_connect (G_OBJECT (priv->cover_button), 
-			  "drag_data_received",
-			  G_CALLBACK (cover_button_drag_data_received), 
-			  info);
+	if (priv->interactive) {
+		priv->cover_button = gtk_button_new ();
+		gtk_button_set_relief (GTK_BUTTON (priv->cover_button),
+				       GTK_RELIEF_NONE);
 
-	priv->cover_image = gtk_image_new_from_stock (GOO_STOCK_NO_COVER, GTK_ICON_SIZE_DIALOG);
-	gtk_widget_set_size_request (priv->cover_image, COVER_SIZE, COVER_SIZE);
+		gtk_tooltips_set_tip (priv->tips,
+				      GTK_WIDGET (priv->cover_button),
+				      _("Click here to choose a cover for this CD"),
+				      NULL);
+		g_signal_connect (G_OBJECT (priv->cover_button),
+				  "clicked",
+				  G_CALLBACK (cover_button_clicked_cb),
+				  info);
+		gtk_drag_dest_set (priv->cover_button,
+				   GTK_DEST_DEFAULT_ALL,
+				   target_table, n_targets,
+				   GDK_ACTION_COPY);
+		g_signal_connect (G_OBJECT (priv->cover_button), 
+				  "drag_data_received",
+				  G_CALLBACK (cover_button_drag_data_received), 
+				  info);
+	}
+
+	priv->cover_image = gtk_image_new_from_stock (GOO_STOCK_NO_DISC, GTK_ICON_SIZE_DIALOG);
+	if (priv->interactive)
+		gtk_widget_set_size_request (priv->cover_image, COVER_SIZE, COVER_SIZE);
+	else
+		gtk_widget_set_size_request (priv->cover_image, TRAY_COVER_SIZE, TRAY_COVER_SIZE);
 	gtk_widget_show (priv->cover_image);
-	gtk_container_add (GTK_CONTAINER (priv->cover_button), priv->cover_image);
+	
+	if (priv->interactive)
+		gtk_container_add (GTK_CONTAINER (priv->cover_button), priv->cover_image);
+	else
+		priv->cover_button = priv->cover_image;
 
-	gtk_box_pack_start (GTK_BOX (info), priv->cover_button, FALSE, FALSE, 0);
+	/* Status image */
+	
+	priv->status_image = gtk_image_new_from_stock (GOO_STOCK_NO_DISC, GTK_ICON_SIZE_DIALOG);
+	if (priv->interactive)
+		gtk_widget_set_size_request (priv->status_image, COVER_SIZE, COVER_SIZE);
+	else
+		gtk_widget_set_size_request (priv->status_image, TRAY_COVER_SIZE, TRAY_COVER_SIZE);
+	gtk_widget_show (priv->cover_image);
+	/*gtk_container_set_border_width (GTK_CONTAINER (priv->status_image), 6);*/
 
-	priv->cover_sep = gtk_vseparator_new ();
-	gtk_box_pack_start (GTK_BOX (info), priv->cover_sep, FALSE, FALSE, 6);
-	gtk_widget_show (priv->cover_sep);
-	gtk_widget_set_no_show_all (priv->cover_sep, TRUE);
+	/* Frame */
 
+	priv->notebook = gtk_notebook_new ();
+	gtk_container_set_border_width (GTK_CONTAINER (priv->notebook), 0);
+	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (priv->notebook), FALSE);
+	gtk_notebook_set_show_border (GTK_NOTEBOOK (priv->notebook), FALSE);
+	gtk_widget_show (priv->notebook);
+	
+	gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), priv->status_image, NULL);
+	gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), priv->cover_button, NULL);
+	
+	priv->cover_frame = gtk_frame_new (NULL);
+	gtk_container_set_border_width (GTK_CONTAINER (priv->cover_frame), 6);
+	if (! priv->interactive)
+		gtk_frame_set_shadow_type (GTK_FRAME (priv->cover_frame), GTK_SHADOW_ETCHED_IN);
+	gtk_widget_show (priv->cover_frame);
+	gtk_container_add (GTK_CONTAINER (priv->cover_frame), priv->notebook);
+
+	gtk_box_pack_start (GTK_BOX (info), priv->cover_frame, FALSE, FALSE, 0);
+		
 	/**/
 
 	vbox = gtk_vbox_new (FALSE, 0);
@@ -548,7 +571,7 @@ goo_player_info_init (GooPlayerInfo *info)
 			  G_CALLBACK (time_scale_value_changed_cb), 
 			  info);
 	g_signal_connect (priv->time_scale, 
-			  "button-press-event",
+			  "button_press_event",
 			  G_CALLBACK (time_scale_button_press_cb), 
 			  info);
 	g_signal_connect (priv->time_scale, 
@@ -572,180 +595,15 @@ goo_player_info_finalize (GObject *object)
 			g_source_remove (info->priv->update_id);
 			info->priv->update_id = 0;
 		}
-		if (info->priv->player != NULL)
-			g_object_unref (info->priv->player);
 		gtk_object_unref (GTK_OBJECT (info->priv->tips));
-		g_free (info->priv);
 		info->priv = NULL;
 	}
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
-}
-
-
-GtkWidget *
-goo_player_info_new (GooWindow *window,
-		     GooPlayer *player)
-{
-	GooPlayerInfo *info;
-	
-	info = GOO_PLAYER_INFO (g_object_new (GOO_TYPE_PLAYER_INFO, NULL));
-
-	if (window != NULL) 
-		info->priv->window = window;
-
-	if (player != NULL) {
-		g_object_ref (player);
-		info->priv->player = player;
-        }
-
-	return GTK_WIDGET (info);
-}
-
-
-void
-goo_player_info_set_player (GooPlayerInfo  *info,
-			    GooPlayer      *player)
-{
-	if (info->priv->player != NULL) {
-		g_object_unref (info->priv->player);
-		info->priv->player = NULL;
-	}
-
-	if (player != NULL) {
-		g_object_ref (player);
-		info->priv->player = player;
-	}
+	G_OBJECT_CLASS (goo_player_info_parent_class)->finalize (object);
 }
 
 
 static void
-update_subtitle (GooPlayerInfo  *info,
-		 SongInfo       *song)
-{
-	GooPlayerInfoPrivateData *priv = info->priv;
-	const char *title, *subtitle;
-
-	title = goo_player_get_title (priv->player);
-	subtitle = goo_player_get_subtitle (priv->player);
-
-	if ((title == NULL) || (subtitle == NULL)) {
-		set_time_string (priv->total_time, song->time);
-		set_title2 (info, priv->total_time);
-	} else {
-		set_title2 (info, subtitle);
-		set_title3 (info, title);
-		gtk_label_set_selectable (GTK_LABEL (priv->title2_label), TRUE);
-		gtk_label_set_selectable (GTK_LABEL (priv->title3_label), TRUE);
-	}
-}
-
-
-void
-goo_player_info_update_state (GooPlayerInfo  *info)
-{
-	GooPlayerInfoPrivateData *priv = info->priv;
-	GooPlayerState state;
-
-	if (priv->player == NULL)
-		return;
-
-	state = goo_player_get_state (priv->player);
-
-	if ((state == GOO_PLAYER_STATE_PLAYING) 
-	    || (state == GOO_PLAYER_STATE_PAUSED)) {
-		gtk_widget_show (priv->time_scale);
-		gtk_widget_show (priv->time_label);
-		gtk_widget_show (priv->playing_label);
-	} else {
-		gtk_widget_hide (priv->time_scale);
-		gtk_widget_hide (priv->time_label);
-		gtk_widget_hide (priv->playing_label);
-	}
-
-	gtk_label_set_selectable (GTK_LABEL (priv->title1_label), FALSE);
-	gtk_label_set_selectable (GTK_LABEL (priv->title2_label), FALSE);
-	gtk_label_set_selectable (GTK_LABEL (priv->title3_label), FALSE);
-
-	if (state == GOO_PLAYER_STATE_ERROR) {
-		GError *error = goo_player_get_error (priv->player);
-		set_title1 (info, error->message);
-		set_title2 (info, "");
-		set_title3 (info, "");
-		g_error_free (error);
-
-	} else {
-		SongInfo *song = goo_player_get_song (priv->player, goo_player_get_current_song (priv->player));
-
-		if (song != NULL) {
-			char *state_s = "";
-
-			priv->song_length = song->time;
-
-			if (state == GOO_PLAYER_STATE_PAUSED)
-				state_s = _("Paused");
-			set_playing (info, state_s);
-
-			set_title1 (info, song->title);
-			gtk_label_set_selectable (GTK_LABEL (priv->title1_label), TRUE);
-
-			update_subtitle (info, song);
-
-			song_info_unref (song);
-
-		} else if (state == GOO_PLAYER_STATE_EJECTING) {
-			set_title1 (info, _("Ejecting CD"));
-			set_title2 (info, "");
-			
-		} else if (state == GOO_PLAYER_STATE_UPDATING) {
-			set_title1 (info, _("Checking CD drive"));
-			set_title2 (info, "");
-			
-		} else if (state == GOO_PLAYER_STATE_SEEKING) {
-			set_title1 (info, _("Reading CD"));
-			set_title2 (info, "");
-
-		} else if (state == GOO_PLAYER_STATE_LISTING) {
-			set_title1 (info, _("Reading CD"));
-			set_title2 (info, "");
-
-		} else {
-			const char *title, *subtitle;
-
-			title = goo_player_get_title (priv->player);
-			subtitle = goo_player_get_subtitle (priv->player);
-
-			if (title != NULL) {
-				set_title1 (info, title);
-				gtk_label_set_selectable (GTK_LABEL (priv->title1_label), TRUE);
-			} else
-				set_title1 (info, _("Audio CD"));
-
-			if (subtitle != NULL) {
-				set_title2 (info, subtitle);
-				set_title3 (info, priv->total_time);
-				gtk_label_set_selectable (GTK_LABEL (priv->title2_label), TRUE);
-			} else {
-				set_title2 (info, priv->total_time);
-				set_title3 (info, "");
-			}
-		}
-	}
-}
-
-
-void
-goo_player_info_set_total_time (GooPlayerInfo  *info,
-				gint64          total_time)
-{
-	GooPlayerInfoPrivateData *priv = info->priv;
-
-	set_time_string (priv->total_time, total_time);
-	goo_player_info_update_state (info);
-}
-
-
-void
 goo_player_info_set_time (GooPlayerInfo  *info,
 			  gint64          current_time)
 {
@@ -759,12 +617,174 @@ goo_player_info_set_time (GooPlayerInfo  *info,
 	set_time (info, priv->time);
 
 	g_signal_handlers_block_by_data (priv->time_scale, info);
-	gtk_range_set_value (GTK_RANGE (priv->time_scale), (double) current_time / priv->song_length);
+	gtk_range_set_value (GTK_RANGE (priv->time_scale), (double) current_time / priv->track_length);
 	g_signal_handlers_unblock_by_data (priv->time_scale, info);
 }
 
 
-void
+static gboolean
+update_progress_cb (gpointer data)
+{
+	GooPlayerInfo *info = data;
+
+	if ((info->priv->fraction < 0.0) || (info->priv->fraction > 1.0))
+		/* nothing */;
+	else
+		goo_player_info_set_time (info, info->priv->fraction * info->priv->track_length);
+	
+	return FALSE;
+}
+
+
+static void
+player_progress_cb (GooPlayer     *player,
+		    double         fraction,
+		    GooPlayerInfo *info)
+{
+	info->priv->fraction = fraction;
+	info->priv->update_progress_timeout = g_idle_add (update_progress_cb, info);
+}
+
+
+double
+goo_player_info_get_progress (GooPlayerInfo *info)
+{
+	return info->priv->fraction;
+}
+
+
+static void
+update_subtitle (GooPlayerInfo *info,
+		 TrackInfo     *track)
+{
+ 	AlbumInfo *album;
+	
+	album = goo_window_get_album (info->priv->window);
+	
+	if ((album->title == NULL) || (album->artist == NULL)) {
+		set_time_string (info->priv->total_time, track->length);
+		set_title2 (info, info->priv->total_time);
+	} 
+	else {
+		set_title2 (info, album->title);
+		set_title3 (info, album->artist);
+		gtk_label_set_selectable (GTK_LABEL (info->priv->title2_label), info->priv->interactive);
+		gtk_label_set_selectable (GTK_LABEL (info->priv->title3_label), info->priv->interactive);
+	}
+}
+
+
+static void
+goo_player_info_update_state (GooPlayerInfo  *info)
+{
+	GooPlayerInfoPrivateData *priv = info->priv;
+	GooPlayerState  state;
+	AlbumInfo      *album;
+	GooPlayer      *player;
+	
+	if (info->priv->window == NULL)
+		return;
+	player = goo_window_get_player (info->priv->window);
+	if (player == NULL)
+		return;
+
+	state = goo_player_get_state (player);
+	album = goo_window_get_album (info->priv->window);
+
+	if ((state == GOO_PLAYER_STATE_PLAYING) 
+	    || (state == GOO_PLAYER_STATE_PAUSED)) {
+	    	if (info->priv->interactive)
+			gtk_widget_show (priv->time_scale);
+		gtk_widget_show (priv->time_label);
+		gtk_widget_show (priv->playing_label);
+	} 
+	else {
+		gtk_widget_hide (priv->time_scale);
+		gtk_widget_hide (priv->time_label);
+		gtk_widget_hide (priv->playing_label);
+	}
+
+	gtk_label_set_selectable (GTK_LABEL (priv->title1_label), FALSE);
+	gtk_label_set_selectable (GTK_LABEL (priv->title2_label), FALSE);
+	gtk_label_set_selectable (GTK_LABEL (priv->title3_label), FALSE);
+
+	if ((state == GOO_PLAYER_STATE_ERROR)
+	    || (state == GOO_PLAYER_STATE_NO_DISC)
+	    || (state == GOO_PLAYER_STATE_DATA_DISC)) {
+		GError *error = goo_player_get_error (player);
+		
+		set_title1 (info, error->message);
+		set_title2 (info, "");
+		set_title3 (info, "");
+		g_error_free (error);
+	} 
+	else {
+		TrackInfo *track;
+		
+		track = album_info_get_track (album, goo_player_get_current_track (player));
+
+		if (track != NULL) {
+			char *state_s = "";
+
+			priv->track_length = track->length;
+
+			if (state == GOO_PLAYER_STATE_PAUSED)
+				state_s = _("Paused");
+			set_playing (info, state_s);
+
+			set_title1 (info, track->title);
+			gtk_label_set_selectable (GTK_LABEL (priv->title1_label), info->priv->interactive);
+
+			update_subtitle (info, track);
+		} 
+		else if (state == GOO_PLAYER_STATE_EJECTING) {
+			set_title1 (info, _("Ejecting CD"));
+			set_title2 (info, "");
+		} 
+		else if (state == GOO_PLAYER_STATE_UPDATING) {
+			set_title1 (info, _("Checking CD drive"));
+			set_title2 (info, "");
+		} 
+		else if (state == GOO_PLAYER_STATE_SEEKING) {
+			set_title1 (info, _("Reading CD"));
+			set_title2 (info, "");
+		} 
+		else if (state == GOO_PLAYER_STATE_LISTING) {
+			set_title1 (info, _("Reading CD"));
+			set_title2 (info, "");
+		} 
+		else {
+			char year[128];
+			
+			if (g_date_valid (album->release_date) != 0)
+				sprintf (year, "%u", g_date_get_year (album->release_date));
+			else
+				year[0] = '\0';
+
+			if (album->title != NULL) {
+				set_title1 (info, album->title);
+				gtk_label_set_selectable (GTK_LABEL (priv->title1_label), info->priv->interactive);
+			} 
+			else
+				set_title1 (info, _("Audio CD"));
+
+			if (album->artist != NULL) {
+				set_title2 (info, album->artist);
+				set_title3 (info, year);
+				gtk_label_set_selectable (GTK_LABEL (priv->title2_label), info->priv->interactive);
+			} 
+			else {
+				set_title2 (info, year);
+				set_title3 (info, "");
+			}
+		}
+		
+		track_info_unref (track);
+	}
+}
+
+
+static void
 goo_player_info_set_sensitive (GooPlayerInfo  *info,
 			       gboolean        value)
 {
@@ -772,15 +792,183 @@ goo_player_info_set_sensitive (GooPlayerInfo  *info,
 }
 
 
-void
-goo_player_info_set_cover (GooPlayerInfo  *info,
-			   GdkPixbuf      *cover)
+static void
+player_state_changed_cb (GooPlayer     *player,
+			 GooPlayerInfo *info)
 {
-	if (cover != NULL) {
-		gtk_image_set_from_pixbuf (GTK_IMAGE (info->priv->cover_image), cover);
-		gtk_widget_hide (info->priv->cover_sep);
-	} else {
-		gtk_image_set_from_stock (GTK_IMAGE (info->priv->cover_image), GOO_STOCK_NO_COVER, GTK_ICON_SIZE_DIALOG);
-		gtk_widget_show (info->priv->cover_sep);
+	goo_player_info_update_state (info);
+	goo_player_info_set_sensitive (info, (goo_player_get_state (player) != GOO_PLAYER_STATE_ERROR) && (goo_player_get_discid (player) != NULL));
+}
+
+
+static void
+player_start_cb (GooPlayer       *player,
+		 GooPlayerAction  action,
+		 GooPlayerInfo   *info)
+{
+	goo_player_info_update_state (info);
+}
+
+
+static void
+goo_player_info_set_total_time (GooPlayerInfo  *info,
+				gint64          total_time)
+{
+	GooPlayerInfoPrivateData *priv = info->priv;
+
+	set_time_string (priv->total_time, total_time);
+	goo_player_info_update_state (info);
+}
+
+
+static void
+player_done_cb (GooPlayer       *player,
+		GooPlayerAction  action,
+		GError          *error,
+		GooPlayerInfo   *info)
+{
+	AlbumInfo *album;
+	
+	switch (action) {
+	case GOO_PLAYER_ACTION_LIST:
+		goo_player_info_update_state (info);
+		album = goo_player_get_album (player);
+		goo_player_info_set_total_time (info, album->total_length);
+		break;
+	case GOO_PLAYER_ACTION_METADATA:
+		goo_player_info_update_state (info);
+		break;
+	case GOO_PLAYER_ACTION_SEEK_SONG:
+		goo_player_info_update_state (info);
+		album = goo_player_get_album (player);
+		goo_player_info_set_total_time (info, album_info_get_track (album, goo_player_get_current_track (player))->length);
+		break;
+	case GOO_PLAYER_ACTION_PLAY:
+	case GOO_PLAYER_ACTION_STOP:
+	case GOO_PLAYER_ACTION_EJECT:
+		goo_player_info_set_time (info, 0);
+		break;
+	default:
+		break;
 	}
 }
+
+
+static void
+goo_player_info_set_cover (GooPlayerInfo *info,
+			   const char    *cover)
+{
+	gboolean cover_set = FALSE;
+	
+	if (cover == NULL)
+		return;
+		
+	if (strcmp (cover, "no-disc") == 0) {
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (info->priv->notebook), 0);
+		gtk_image_set_from_stock (GTK_IMAGE (info->priv->status_image), 
+					  GOO_STOCK_NO_DISC, 
+					  GTK_ICON_SIZE_DIALOG);
+	}
+	else if (strcmp (cover, "data-disc") == 0) {
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (info->priv->notebook), 0);
+		gtk_image_set_from_stock (GTK_IMAGE (info->priv->status_image), 
+					  GOO_STOCK_DATA_DISC, 
+					  GTK_ICON_SIZE_DIALOG);
+	}
+	else if (strcmp (cover, "audio-cd") == 0) {
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (info->priv->notebook), 1);
+		gtk_image_set_from_stock (GTK_IMAGE (info->priv->cover_image), 
+					  GOO_STOCK_AUDIO_CD, 
+					  GTK_ICON_SIZE_DIALOG);
+	}
+	else {
+		GdkPixbuf *image;
+			
+		image = gdk_pixbuf_new_from_file_at_size (cover, 80, 80, NULL);
+		if (image != NULL) {
+			gtk_notebook_set_current_page (GTK_NOTEBOOK (info->priv->notebook), 1);
+			gtk_image_set_from_pixbuf (GTK_IMAGE (info->priv->cover_image), image);
+			cover_set = TRUE;
+			g_object_unref (image);
+		}
+		else 
+			goo_player_info_set_cover (info, "audio-cd");
+	}
+	
+	if (! info->priv->interactive)
+		gtk_frame_set_shadow_type (GTK_FRAME (info->priv->cover_frame), cover_set ? GTK_SHADOW_NONE : GTK_SHADOW_ETCHED_IN);
+}
+
+
+static void
+window_update_cover_cb (GooWindow     *window,
+			GooPlayerInfo *info)
+{
+	GooPlayerState  state;
+	char           *filename;
+	
+	state = goo_player_get_state (goo_window_get_player (window));
+
+	if ((state == GOO_PLAYER_STATE_ERROR) || (state == GOO_PLAYER_STATE_NO_DISC)) {
+	    	goo_player_info_set_cover (info, "no-disc");
+	    	return;
+	}
+
+	if (state == GOO_PLAYER_STATE_DATA_DISC) { 
+	    	goo_player_info_set_cover (info, "data-disc");
+	    	return;
+	}
+	
+	filename = goo_window_get_cover_filename (window);
+	if (filename == NULL) {
+		goo_player_info_set_cover (info, "audio-cd"); 
+		return;
+	}
+	
+	goo_player_info_set_cover (info, filename);
+	g_free (filename);
+}
+
+
+GtkWidget *
+goo_player_info_new (GooWindow *window,
+		     gboolean   interactive)
+{
+	GooPlayerInfo *info;
+	GooPlayer     *player;
+	
+	g_return_val_if_fail (window != NULL, NULL);
+	
+	player = goo_window_get_player (window);
+	g_return_val_if_fail (player != NULL, NULL);
+	
+	info = GOO_PLAYER_INFO (g_object_new (GOO_TYPE_PLAYER_INFO, NULL));
+
+	info->priv->window = window;
+	info->priv->interactive = interactive;
+	goo_player_info_construct (info);
+	
+	g_signal_connect (window, 
+			  "update_cover",
+			  G_CALLBACK (window_update_cover_cb), 
+			  info);
+	
+	g_signal_connect (player, 
+			  "start",
+			  G_CALLBACK (player_start_cb), 
+			  info);
+	g_signal_connect (player, 
+			  "done",
+			  G_CALLBACK (player_done_cb), 
+			  info);	
+	g_signal_connect (player, 
+			  "progress",
+			  G_CALLBACK (player_progress_cb), 
+			  info);
+	g_signal_connect (player, 
+			  "state_changed",
+			  G_CALLBACK (player_state_changed_cb), 
+			  info);
+
+	return GTK_WIDGET (info);
+}
