@@ -130,7 +130,6 @@ enum {
 static guint goo_window_signals[LAST_SIGNAL] = { 0 };
 
 static int icon_size = 0;
-static GnomeAppClass *parent_class = NULL;
 GList *window_list = NULL;
 
 
@@ -144,6 +143,11 @@ enum {
 	NUMBER_OF_COLUMNS
 };
 
+#define GOO_WINDOW_GET_PRIVATE_DATA(object) \
+	(G_TYPE_INSTANCE_GET_PRIVATE ((object), GOO_TYPE_WINDOW, GooWindowPrivateData))
+
+
+G_DEFINE_TYPE (GooWindow, goo_window, GNOME_TYPE_APP)
 
 static void
 set_active (GooWindow  *window,
@@ -282,6 +286,7 @@ window_update_sensitivity (GooWindow *window)
 	gtk_widget_set_sensitive (priv->list_view, audio_cd);
 
 	set_sensitive (window, "Eject", ! priv->hibernate);
+	set_sensitive (window, "EjectToolBar", ! priv->hibernate);
 	set_sensitive (window, "Preferences", ! priv->hibernate);
 }
 
@@ -510,6 +515,7 @@ goo_window_update_titles (GooWindow *window)
 			track_info_ref (new_track);
 			priv->current_track = new_track;
 		}
+		track_info_unref (new_track);
 
 	} while (gtk_tree_model_iter_next (model, &iter));
 }
@@ -543,10 +549,12 @@ goo_window_finalize (GObject *object)
 			g_source_remove (priv->tray_tooltip_delay);
 			priv->tray_tooltip_delay = 0;
 		}
+		
 		if (priv->next_timeout_handle != 0) {
 			g_source_remove (priv->next_timeout_handle);
 			priv->next_timeout_handle = 0;
 		}
+		
 		if (priv->update_timeout_handle != 0) {
 			g_source_remove (priv->update_timeout_handle);
 			priv->update_timeout_handle = 0;
@@ -574,11 +582,10 @@ goo_window_finalize (GObject *object)
 		path_list_free (priv->url_list);
 		priv->url_list = NULL;
 
-		g_free (window->priv);
 		window->priv = NULL;
 	}
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (goo_window_parent_class)->finalize (object);
 }
 
 
@@ -688,7 +695,7 @@ menu_item_select_cb (GtkMenuItem *proxy,
         GtkAction *action;
         char      *message;
 
-        action = g_object_get_data (G_OBJECT (proxy),  "gtk-action");
+        action = gtk_widget_get_action (GTK_WIDGET (proxy));
         g_return_if_fail (action != NULL);
 
         g_object_get (G_OBJECT (action), "tooltip", &message, NULL);
@@ -1029,7 +1036,7 @@ goo_window_unrealize (GtkWidget *widget)
 	preferences_set_sort_method (window->priv->sort_method);
 	preferences_set_sort_type (window->priv->sort_type);
 
-	GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+	GTK_WIDGET_CLASS (goo_window_parent_class)->unrealize (widget);
 }
 
 
@@ -1053,7 +1060,7 @@ goo_window_show (GtkWidget *widget)
 	gboolean   view_foobar;
 
 	if (! HideShow) 
-		GTK_WIDGET_CLASS (parent_class)->show (widget);
+		GTK_WIDGET_CLASS (goo_window_parent_class)->show (widget);
 	else
 		HideShow = FALSE;
 
@@ -1076,7 +1083,6 @@ goo_window_class_init (GooWindowClass *class)
 	GObjectClass   *gobject_class;
 	GtkWidgetClass *widget_class;
 
-	parent_class = g_type_class_peek_parent (class);
 	widget_class = (GtkWidgetClass*) class;
 	gobject_class = (GObjectClass*) class;
 
@@ -1093,6 +1099,7 @@ goo_window_class_init (GooWindowClass *class)
 			      goo_marshal_VOID__VOID,
 			      G_TYPE_NONE, 
 			      0);
+	g_type_class_add_private (class, sizeof (GooWindowPrivateData));
 }
 
 
@@ -1445,7 +1452,7 @@ window_update_title (GooWindow *window)
 	else if (state == GOO_PLAYER_STATE_ERROR) {
 		GError *error = goo_player_get_error (priv->player);
 		
-		g_string_append (title, g_strdup (error->message));
+		g_string_append (title, error->message);
 		g_error_free (error);
 	} 
 	else if (state == GOO_PLAYER_STATE_NO_DISC) {
@@ -1566,6 +1573,7 @@ save_config_file (GKeyFile *kv_file,
 	else
 		g_error_free (error);
 
+	g_free (buffer);
 	gnome_vfs_close (handle);
 }
 
@@ -1659,7 +1667,7 @@ auto_fetch_cover_image (GooWindow *window)
 		g_free (filename);
 		return;
 	}
-
+	g_free (filename);
 	if (window->priv->album->asin != NULL) 
 		fetch_cover_image_from_asin (window, window->priv->album->asin);
 	else if ((window->priv->album->title != NULL) && (window->priv->album->artist != NULL))
@@ -2204,7 +2212,7 @@ volume_button_changed_cb (GooVolumeToolButton *button,
 static void
 goo_window_init (GooWindow *window)
 {
-	window->priv = g_new0 (GooWindowPrivateData, 1);
+	window->priv = GOO_WINDOW_GET_PRIVATE_DATA (window);
 	window->priv->exiting = FALSE;
 	window->priv->check_id = 0;
 	window->priv->url_list = NULL;
@@ -2645,35 +2653,6 @@ goo_window_construct (GooWindow  *window,
 					   window);
 }
 
-
-GType
-goo_window_get_type ()
-{
-        static GType type = 0;
-
-        if (! type) {
-                GTypeInfo type_info = {
-			sizeof (GooWindowClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) goo_window_class_init,
-			NULL,
-			NULL,
-			sizeof (GooWindow),
-			0,
-			(GInstanceInitFunc) goo_window_init
-		};
-
-		type = g_type_register_static (GNOME_TYPE_APP,
-					       "GooWindow",
-					       &type_info,
-					       0);
-	}
-
-        return type;
-}
-
-
 GtkWindow * 
 goo_window_new (const char *device)
 {
