@@ -38,6 +38,7 @@
 #include "goo-application.h"
 #include "gtk-utils.h"
 #include "glib-utils.h"
+#include "cd-drive.h"
 
 #ifdef HAVE_MMKEYS
 #include <X11/Xlib.h>
@@ -46,13 +47,16 @@
 #endif /* HAVE_MMKEYS */
 
 #ifdef HAVE_LIBNOTIFY
+
 #include <libnotify/notify.h>
-static NotifyHandle *notify_h = NULL;
+static NotifyNotification *notification = NULL;
+
 #endif /* HAVE_LIBNOTIFY */
 
 GtkWindow *main_window = NULL;
 int        AutoPlay = FALSE;
 int        PlayPause = FALSE;
+int        Stop = FALSE;
 int        Next = FALSE;
 int        Prev = FALSE;
 int        Eject = FALSE;
@@ -60,8 +64,9 @@ int        HideShow = FALSE;
 int        VolumeUp = FALSE;
 int        VolumeDown = FALSE;
 int        Quit = FALSE;
+GList     *Drives = NULL;
 
-static void     prepare_app         (poptContext pctx);
+static void     prepare_app         (void);
 static void     initialize_data     (void);
 static void     release_data        (void);
 
@@ -70,80 +75,108 @@ static gboolean session_is_restored (void);
 static gboolean load_session        (void);
 static void     init_mmkeys         (void);
 
-static char         *default_device = NULL;
-static BonoboObject *goo_application = NULL;
+static char           *default_device = NULL;
+static BonoboObject   *goo_application = NULL;
+static GOptionContext *context = NULL;
 
-struct poptOption options[] = {
-	{ "device", 'd', POPT_ARG_STRING, &default_device, 0,
+static const GOptionEntry options[] = {
+	{ "device", 'd',  0, G_OPTION_ARG_STRING, &default_device, 
 	  N_("CD device to be used"),
 	  N_("DEVICE_PATH") },
-	{ "play", '\0', POPT_ARG_NONE, &AutoPlay, 0,
+	{ "play", '\0', 0, G_OPTION_ARG_NONE, &AutoPlay,
           N_("Play the CD on startup"),
           0 },
-	{ "play-pause", '\0', POPT_ARG_NONE, &PlayPause, 0,
+	{ "play-pause", '\0', 0, G_OPTION_ARG_NONE, &PlayPause, 
           N_("Play/Pause"),
           0 },
-	{ "next", '\0', POPT_ARG_NONE, &Next, 0,
+        { "stop", '\0', 0, G_OPTION_ARG_NONE, &Stop, 
+          N_("Stop playing"),
+          0 },
+	{ "next", '\0', 0, G_OPTION_ARG_NONE, &Next,
           N_("Play the next track"),
           0 },
-	{ "previous", '\0', POPT_ARG_NONE, &Prev, 0,
+	{ "previous", '\0', 0, G_OPTION_ARG_NONE, &Prev,
           N_("Play the previous track"),
           0 },
-	{ "eject", '\0', POPT_ARG_NONE, &Eject, 0,
+	{ "eject", '\0', 0, G_OPTION_ARG_NONE, &Eject,
           N_("Eject the CD"),
           0 },
-	{ "hide-show", '\0', POPT_ARG_NONE, &HideShow, 0,
+	{ "hide-show", '\0', 0, G_OPTION_ARG_NONE, &HideShow,
           N_("Hide/Show the main window"),
           0 },
-	{ "volume-up", '\0', POPT_ARG_NONE, &VolumeUp, 0,
+	{ "volume-up", '\0', 0, G_OPTION_ARG_NONE, &VolumeUp,
           N_("Volume Up"),
           0 },
-	{ "volume-down", '\0', POPT_ARG_NONE, &VolumeDown, 0,
+	{ "volume-down", '\0', 0, G_OPTION_ARG_NONE, &VolumeDown,
           N_("Volume Down"),
           0 },
-	{ "quit", '\0', POPT_ARG_NONE, &Quit, 0,
+	{ "quit", '\0', 0, G_OPTION_ARG_NONE, &Quit,
           N_("Quit the application"),
           0 },
-	{ NULL, '\0', 0, NULL, 0 }
+          
+	{ NULL }
 };
 
 
 /* -- Main -- */
 
-int main (int argc, char **argv)
+
+int main (int    argc, 
+	  char **argv)
 {
 	GnomeProgram *program;
-	GValue value = { 0 };
-	poptContext pctx;
-	CORBA_Object factory;
+	char         *description;
+	CORBA_Object  factory;
+
+	if (! g_thread_supported ()) 
+		g_thread_init (NULL);
 
 	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
-
- 	program = gnome_program_init ("goobox", VERSION,
-				      LIBGNOMEUI_MODULE, argc, argv,
-				      GNOME_PARAM_POPT_TABLE, options,
-				      GNOME_PARAM_HUMAN_READABLE_NAME, _("CD player and ripper"),
+	
+        description = g_strdup_printf ("- %s", _("Play CDs and save the tracks to disk as files"));
+        context = g_option_context_new (description);
+        g_free (description);
+        g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+	g_option_context_add_group (context, gst_init_get_option_group ());
+  	g_option_context_set_ignore_unknown_options (context, TRUE);
+  
+	program = gnome_program_init ("goobox", VERSION,
+			              LIBGNOMEUI_MODULE, argc, argv,
+			              GNOME_PARAM_GOPTION_CONTEXT, context,
+			              GNOME_PARAM_HUMAN_READABLE_NAME, _("CD Player"),
 				      GNOME_PARAM_APP_PREFIX, GOO_PREFIX,
                                       GNOME_PARAM_APP_SYSCONFDIR, GOO_SYSCONFDIR,
                                       GNOME_PARAM_APP_DATADIR, GOO_DATADIR,
                                       GNOME_PARAM_APP_LIBDIR, GOO_LIBDIR,
 				      NULL);
 
-	g_object_get_property (G_OBJECT (program),
-			       GNOME_PARAM_POPT_CONTEXT,
-			       g_value_init (&value, G_TYPE_POINTER));
-	pctx = g_value_get_pointer (&value);
+        if (! g_thread_supported ()) {
+                g_thread_init (NULL);
+                gdk_threads_init ();
+        }
+
+	if (! gnome_vfs_init ()) 
+                g_error ("Cannot initialize the Virtual File System.");
+	gnome_authentication_manager_init ();
+	glade_gnome_init ();
+	
+#ifdef HAVE_LIBNOTIFY
+
+	if (! notify_init ("goobox")) 
+                g_warning ("Cannot initialize notification system.");
+                
+#endif /* HAVE_LIBNOTIFY */
 
 	factory = bonobo_activation_activate_from_id ("OAFIID:GNOME_Goobox_Application_Factory",
 						      Bonobo_ACTIVATION_FLAG_EXISTING_ONLY,
 						      NULL, NULL);
 
 	if (factory != NULL) {
-		CORBA_Environment     env;
+		CORBA_Environment        env;
 		GNOME_Goobox_Application app;
-
+		
 		CORBA_exception_init (&env);
 
 		app = bonobo_activation_activate_from_id ("OAFIID:GNOME_Goobox_Application", 0, NULL, &env);
@@ -152,6 +185,8 @@ int main (int argc, char **argv)
                 	GNOME_Goobox_Application_play (app, &env);
 		else if (PlayPause)
                 	GNOME_Goobox_Application_play_pause (app, &env);
+                else if (Stop)
+                	GNOME_Goobox_Application_stop (app, &env);
 		else if (Next)
                 	GNOME_Goobox_Application_next (app, &env);
 		else if (Prev)
@@ -177,30 +212,10 @@ int main (int argc, char **argv)
 		exit (0);
 	}
 
-	if (! g_thread_supported ()) {
-		g_thread_init (NULL);
-		gdk_threads_init ();
-	}
-
-	if (! gnome_vfs_init ()) 
-                g_error ("Cannot initialize the Virtual File System.");
-
-	glade_gnome_init ();
-	
-	gst_init (NULL, NULL);
-	/*gst_use_threads (TRUE);*/
-
-
-#ifdef HAVE_LIBNOTIFY
-	if (! notify_init ("goobox")) 
-                g_error ("Cannot initialize notification system.");
-#endif /* HAVE_LIBNOTIFY */
-
 	goo_stock_init ();
 	init_session (argv);
 	initialize_data ();
-	prepare_app (pctx);
-	poptFreeContext (pctx);
+	prepare_app ();
 
 	bonobo_main ();
 
@@ -214,25 +229,89 @@ int main (int argc, char **argv)
 
 
 static void 
-initialize_data ()
+initialize_data (void)
 {
-	char *icon_path = PIXMAPSDIR "/goobox.png";
-
-	if (! path_is_file (icon_path))
-                g_warning ("Could not find %s", icon_path);
-	else
-		gnome_window_icon_set_default_from_file (icon_path);
+        g_set_application_name (_("CD Player"));
+        gtk_window_set_default_icon_name ("goobox");
 
 	eel_gconf_monitor_add ("/apps/goobox");
+	
+	Drives = scan_for_cdroms (FALSE, FALSE);
 }
 
 
 static void 
-release_data ()
+release_data (void)
 {
 	if (goo_application != NULL)
 		bonobo_object_unref (goo_application);
 	eel_global_client_free ();
+	
+	if (Drives != NULL) {
+		g_list_foreach (Drives, (GFunc) cd_drive_free, NULL);
+		g_list_free (Drives);
+		Drives = NULL;
+	}
+}
+
+
+CDDrive * 
+get_drive_from_device (const char *device)
+{
+	CDDrive    *result = NULL;
+	char       *resolved_device = NULL;
+	char       *resolved_real_device = NULL;
+	const char *real_device = NULL;
+	GList      *scan;
+	
+	if (device == NULL)
+		return NULL;
+	
+	if (resolve_all_symlinks (device, &resolved_device) != GNOME_VFS_OK)
+		resolved_device = NULL;
+
+	if (resolved_device == NULL)
+		return NULL;
+	
+	device = get_path_from_uri (resolved_device);
+	for (scan = Drives; scan; scan = scan->next) {
+		CDDrive *drive = scan->data;
+		
+		if (drive->device == NULL)
+			continue;
+		if (resolve_all_symlinks (drive->device, &resolved_real_device) != GNOME_VFS_OK)
+			continue;
+		real_device = get_path_from_uri (resolved_real_device);
+		if (strcmp (real_device, device) == 0) {
+			result = drive;
+			break;
+		}
+	}
+	
+	g_free (resolved_device);
+	
+	return result;
+}
+
+
+GtkWindow *
+get_window_from_device (const char *device)
+{
+	CDDrive *device_drive;
+	GList   *scan;
+	
+	device_drive = get_drive_from_device (device);
+	if (device_drive == NULL)
+		return NULL;
+		
+	for (scan = window_list; scan; scan = scan->next) {
+		GooWindow *window = scan->data;
+
+		if (goo_player_get_drive (goo_window_get_player (window)) == device_drive)
+			return (GtkWindow *) window;
+	}
+	
+	return NULL;
 }
 
 
@@ -242,29 +321,37 @@ release_data ()
 static gboolean
 check_plugins (void)
 {
-	GstElement *cdparanoia;
-	gboolean    result;
+	char *required_plugins[] = { "cdparanoiasrc", "gnomevfssink", "audioconvert", "volume" };
+	int   i;
 
-	cdparanoia = gst_element_factory_make ("cdparanoia", "cdreader");
-	result = (cdparanoia != NULL);
-	if (cdparanoia != NULL) 
-		gst_object_unref (GST_OBJECT (cdparanoia));
+	for (i = 0; i < G_N_ELEMENTS (required_plugins); i++) {
+		GstElement *element;
+		gboolean    present;
+		
+		element = gst_element_factory_make (required_plugins[i], NULL);
+		present = (element != NULL);
+		if (element != NULL) 
+			gst_object_unref (GST_OBJECT (element));
+	
+		if (! present)
+			return FALSE;
+	}	
 
-	return result;
+	return TRUE;
 }
 
 
 static void 
-prepare_app (poptContext pctx)
+prepare_app (void)
 {
-	if (!check_plugins()) {
+	if (! check_plugins ()) {
 		GtkWidget *d;
 		d = _gtk_message_dialog_new (NULL,
 					     0,
 					     GTK_STOCK_DIALOG_ERROR,
 					     _("Cannot start the CD player"),
-					     _("In order to read CDs you have to install the cdparanoia gstreamer plugin"),
-					     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+					     _("In order to read CDs you have to install the gstreamer base plugins"),
+					     GTK_STOCK_OK, GTK_RESPONSE_OK,
 					     NULL);
 		g_signal_connect (G_OBJECT (d), "response",
 				  G_CALLBACK (bonobo_main_quit),
@@ -273,7 +360,7 @@ prepare_app (poptContext pctx)
 
 		return;
 	}
-
+	
 	if (session_is_restored ()) 
 		load_session ();
 	else 
@@ -288,7 +375,7 @@ prepare_app (poptContext pctx)
 }
 
 
- /* SM support */
+/* SM support */
 
 
 /* The master client we use for SM */
@@ -306,7 +393,7 @@ save_session (GnomeClient *client)
 	prefix = gnome_client_get_config_prefix (client);
 	gnome_config_push_prefix (prefix);
 
-	gnome_config_set_string ("Session/location", goo_player_get_location (goo_window_get_player (GOO_WINDOW (main_window))));
+	gnome_config_set_string ("Session/device", goo_player_get_device (goo_window_get_player (GOO_WINDOW (main_window))));
 
 	gnome_config_pop_prefix ();
 	gnome_config_sync ();
@@ -398,13 +485,13 @@ session_is_restored (void)
 gboolean
 load_session (void)
 {
-	char *location;
+	char *device;
 
 	gnome_config_push_prefix (gnome_client_get_config_prefix (master_client));
 
-	location = gnome_config_get_string ("Session/location");
-	main_window = goo_window_new (location);
-	g_free (location);
+	device = gnome_config_get_string ("Session/device");
+	main_window = goo_window_new (device);
+	g_free (device);
 
 	gnome_config_pop_prefix ();
 
@@ -486,30 +573,29 @@ filter_mmkeys (GdkXEvent *xevent,
 	if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPlay) == key->keycode) {	
 		goo_window_toggle_play (GOO_WINDOW (main_window));
 		return GDK_FILTER_REMOVE;
-
-	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPause) == key->keycode) {	
+	} 
+	else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPause) == key->keycode) {	
 		goo_window_pause (GOO_WINDOW (main_window));
 		return GDK_FILTER_REMOVE;
-
-	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioStop) == key->keycode) {
+	} 
+	else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioStop) == key->keycode) {
 		goo_window_stop (GOO_WINDOW (main_window));
 		return GDK_FILTER_REMOVE;		
-
-	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPrev) == key->keycode) {
+	} 
+	else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPrev) == key->keycode) {
 		goo_window_prev (GOO_WINDOW (main_window));
 		return GDK_FILTER_REMOVE;		
-
-	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioNext) == key->keycode) {
+	} 
+	else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioNext) == key->keycode) {
 		goo_window_next (GOO_WINDOW (main_window));
 		return GDK_FILTER_REMOVE;
-
-	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_Eject) == key->keycode) {
+	} 
+	else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_Eject) == key->keycode) {
 		goo_window_eject (GOO_WINDOW (main_window));
 		return GDK_FILTER_REMOVE;
-
-	} else {
+	} 
+	else 
 		return GDK_FILTER_CONTINUE;
-	}
 }
 
 
@@ -550,34 +636,116 @@ init_mmkeys (void)
 #endif /* HAVE_MMKEYS */
 
 
+#ifdef HAVE_LIBNOTIFY
+
+
+static gboolean
+play_next (gpointer user_data)
+{
+	GooWindow *window = user_data;
+	
+	goo_window_next (window);
+	return FALSE;
+}
+
+
+static void
+notify_action_next_cb (NotifyNotification *notification,
+                       char               *action,
+                       gpointer            user_data)
+{
+	GooWindow *window = user_data;
+	
+	notify_notification_close (notification, NULL);
+	g_idle_add (play_next, window);
+}
+
+
+static void
+notify_action_stop_cb (NotifyNotification *notification,
+                       char               *action,
+                       gpointer            user_data)
+{
+	GooWindow *window = user_data;
+	
+	goo_window_stop (window);
+	notify_notification_close (notification, NULL);
+}
+
+
+#endif /* HAVE_LIBNOTIFY */
+
+
 void 
-system_notify (const char *title,
-	       const char *msg,
-	       int         x,
-	       int         y)
+system_notify (GooWindow  *window,
+	       const char *title,
+	       const char *msg)
 {
 #ifdef HAVE_LIBNOTIFY
-	NotifyIcon  *icon = notify_icon_new_from_uri("goobox");
-	NotifyHints *hints = NULL;
+        GtkWidget *tray_icon;
+	GdkScreen *screen = NULL;
+	int        x = -1, y = -1;
+	gboolean   supports_actions = FALSE;
+	GList     *caps = NULL;
+	GList     *c;
 
-	if ((x >= 0) && (y >= 0)) {
-		hints = notify_hints_new ();
-		notify_hints_set_int (hints, "x", x);
-		notify_hints_set_int (hints, "y", y);
+	if (! notify_is_initted ())
+		return;
+
+	tray_icon = goo_window_get_tray_icon (window);
+	if (tray_icon != NULL) {
+		GdkWindow *win = tray_icon->window;
+		int        w, h;
+				
+		gdk_window_get_origin (win, &x, &y);
+		gdk_window_get_geometry (win, NULL, NULL, &w, &h, NULL);
+		y += h;
+		x += (w / 2);
+		
+		screen = gtk_widget_get_screen (tray_icon);
 	}
 
-	notify_h = notify_send_notification (notify_h,
-					     "device",
-					     NOTIFY_URGENCY_NORMAL,
-					     title,
-					     msg,
-					     icon,
-					     TRUE, 0, 
-					     hints, // no hints
-					     NULL, // no user data
-					     0);
+	if (notification == NULL) {
+		caps = notify_get_server_caps();
+		if (caps != NULL) {
+			for (c = caps; c != NULL; c = c->next) {
+				if (strcmp ((char*)c->data, "actions") == 0) {
+					supports_actions = TRUE;
+					break;
+				}
+			}
+
+			g_list_foreach (caps, (GFunc)g_free, NULL);
+			g_list_free (caps);
+		}
+
+		notification = notify_notification_new (title, msg, "goobox", NULL);
+		notify_notification_set_urgency (notification, NOTIFY_URGENCY_LOW);
+
+		if (supports_actions) {
+			notify_notification_add_action (notification,
+							GTK_STOCK_MEDIA_NEXT,
+							_("Next"),
+							notify_action_next_cb,
+							window,
+							NULL);
+			notify_notification_add_action (notification,
+							GTK_STOCK_MEDIA_STOP,
+							_("Stop"),
+							notify_action_stop_cb,
+							window,
+							NULL);
+		}
+	}
+	else
+		notify_notification_update (notification, title, msg, "goobox");
+
+	/*if ((x >= 0) && (y >= 0)) 
+		notify_notification_set_geometry_hints (notification,
+							screen,
+							x, y);*/
+
+	notify_notification_show (notification, NULL);
 	
-	if (icon != NULL)
-		notify_icon_destroy (icon);
 #endif /* HAVE_LIBNOTIFY */
 }
